pub struct Options { /* private fields */ }Expand description
Configuration options for the Vampire theorem prover.
Options allow you to configure the behavior of the prover, such as setting time limits. Use the builder pattern to construct options.
Examples
use vampire_prover::Options;
use std::time::Duration;
// Default options (no timeout)
let opts = Options::new();
// Set a timeout
let opts = Options::new().timeout(Duration::from_secs(5));Implementations
Sourceimpl Options
impl Options
Sourcepub fn new() -> Self
pub fn new() -> Self
Creates a new Options with default settings.
By default, no timeout is set.
Examples
use vampire_prover::Options;
let opts = Options::new();Sourcepub fn timeout(self, duration: Duration) -> Self
pub fn timeout(self, duration: Duration) -> Self
Sets the timeout for the prover.
If the prover exceeds this time limit, it will return
[ProofRes::Unknown(UnknownReason::Timeout)].
Arguments
duration- The maximum time the prover should run
Examples
use vampire_prover::Options;
use std::time::Duration;
let opts = Options::new().timeout(Duration::from_secs(10));Trait Implementations